@@ -1,6 +1,6 @@
 /* 
  * Copyright (C) 2006, 2007, 2008 OpenedHand Ltd.
- * Copyright (C) 2009 Nokia Corporation, all rights reserved.
+ * Copyright (C) 2009 Nokia Corporation.
  *
  * Author: Jorn Baayen <jorn@openedhand.com>
  *         Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
@@ -85,8 +85,6 @@ G_DEFINE_TYPE_EXTENDED (GSSDPClient,
                                  gssdp_client_initable_iface_init));
 
 struct _GSSDPClientPrivate {
-        GMainContext      *main_context;
-
         char              *server_id;
         char              *iface;
         char              *host_ip;
@@ -94,6 +92,7 @@ struct _GSSDPClientPrivate {
 
         GSSDPSocketSource *request_socket;
         GSSDPSocketSource *multicast_socket;
+        GSSDPSocketSource *search_socket;
 
         gboolean           active;
 };
@@ -115,10 +114,6 @@ enum {
 
 static guint signals[LAST_SIGNAL];
 
-/* Function prototypes */
-static void
-gssdp_client_set_main_context (GSSDPClient  *client,
-                               GMainContext *context);
 static char *
 make_server_id                (void);
 static gboolean
@@ -130,6 +125,11 @@ multicast_socket_source_cb    (GIOChannel   *source,
                                GIOCondition  condition,
                                gpointer      user_data);
 static gboolean
+search_socket_source_cb       (GIOChannel   *source,
+                               GIOCondition  condition,
+                               gpointer      user_data);
+
+static gboolean
 init_network_info             (GSSDPClient  *client,
                                GError      **error);
 
@@ -210,20 +210,52 @@ gssdp_client_initable_init (GInitable     *initable,
                         (client->priv->multicast_socket,
                          (GSourceFunc) multicast_socket_source_cb,
                          client);
+        } else {
+                goto errors;
         }
 
+        /* Setup send socket. For security reasons, it is not recommended to
+         * send M-SEARCH with source port == SSDP_PORT */
+        client->priv->search_socket = gssdp_socket_source_new
+                                        (GSSDP_SOCKET_SOURCE_TYPE_SEARCH,
+                                         gssdp_client_get_host_ip (client),
+                                         &internal_error);
+        if (client->priv->search_socket != NULL) {
+                gssdp_socket_source_set_callback
+                                        (client->priv->search_socket,
+                                         (GSourceFunc) search_socket_source_cb,
+                                         client);
+        }
  errors:
-        if (!client->priv->request_socket || !client->priv->multicast_socket) {
+        if (!client->priv->request_socket ||
+            !client->priv->multicast_socket ||
+            !client->priv->search_socket) {
                 g_propagate_error (error, internal_error);
 
+                if (client->priv->request_socket) {
+                        g_object_unref (client->priv->request_socket);
+
+                        client->priv->request_socket = NULL;
+                }
+
+                if (client->priv->multicast_socket) {
+                        g_object_unref (client->priv->multicast_socket);
+
+                        client->priv->multicast_socket = NULL;
+                }
+
+                if (client->priv->search_socket) {
+                        g_object_unref (client->priv->search_socket);
+
+                        client->priv->search_socket = NULL;
+                }
+
                 return FALSE;
         }
 
-        gssdp_socket_source_attach (client->priv->request_socket,
-                                    client->priv->main_context);
-
-        gssdp_socket_source_attach (client->priv->multicast_socket,
-                                    client->priv->main_context);
+        gssdp_socket_source_attach (client->priv->request_socket);
+        gssdp_socket_source_attach (client->priv->multicast_socket);
+        gssdp_socket_source_attach (client->priv->search_socket);
 
         return TRUE;
 }
@@ -245,10 +277,12 @@ gssdp_client_get_property (GObject    *object,
                          gssdp_client_get_server_id (client));
                 break;
         case PROP_MAIN_CONTEXT:
+                g_warning ("GSSDPClient:main-context is deprecated."
+                           " Please use g_main_context_push_thread_default()");
                 g_value_set_pointer
                         (value,
                          (gpointer)
-                          gssdp_client_get_main_context (client));
+                          g_main_context_get_thread_default ());
                 break;
         case PROP_IFACE:
                 g_value_set_string (value,
@@ -287,8 +321,8 @@ gssdp_client_set_property (GObject      *object,
                                             g_value_get_string (value));
                 break;
         case PROP_MAIN_CONTEXT:
-                gssdp_client_set_main_context (client,
-                                               g_value_get_pointer (value));
+                g_warning ("GSSDPClient:main-context is deprecated."
+                           " Please use g_main_context_push_thread_default()");
                 break;
         case PROP_IFACE:
                 client->priv->iface = g_value_dup_string (value);
@@ -323,10 +357,9 @@ gssdp_client_dispose (GObject *object)
                 client->priv->multicast_socket = NULL;
         }
 
-        /* Unref the context */
-        if (client->priv->main_context) {
-                g_main_context_unref (client->priv->main_context);
-                client->priv->main_context = NULL;
+        if (client->priv->search_socket) {
+                g_object_unref (client->priv->search_socket);
+                client->priv->search_socket = NULL;
         }
 
         G_OBJECT_CLASS (gssdp_client_parent_class)->dispose (object);
@@ -355,12 +388,12 @@ gssdp_client_class_init (GSSDPClientClass *klass)
 {
         GObjectClass *object_class;
 
-	object_class = G_OBJECT_CLASS (klass);
+        object_class = G_OBJECT_CLASS (klass);
 
-	object_class->set_property = gssdp_client_set_property;
-	object_class->get_property = gssdp_client_get_property;
-	object_class->dispose      = gssdp_client_dispose;
-	object_class->finalize     = gssdp_client_finalize;
+        object_class->set_property = gssdp_client_set_property;
+        object_class->get_property = gssdp_client_get_property;
+        object_class->dispose      = gssdp_client_dispose;
+        object_class->finalize     = gssdp_client_finalize;
 
         g_type_class_add_private (klass, sizeof (GSSDPClientPrivate));
 
@@ -385,6 +418,7 @@ gssdp_client_class_init (GSSDPClientClass *klass)
          * GSSDPClient:main-context
          *
          * The #GMainContext to use. Set to NULL to use the default.
+         * Deprecated: 0.11.2: Use g_main_context_push_thread_default().
          **/
         g_object_class_install_property
                 (object_class,
@@ -504,7 +538,8 @@ gssdp_client_class_init (GSSDPClientClass *klass)
 
 /**
  * gssdp_client_new
- * @main_context: The #GMainContext to associate with, or NULL
+ * @main_context: Deprecated: 0.11.2: Always set to NULL. If you want to
+ *                specify a context use g_main_context_push_thread_default()
  * @iface: The name of the network interface, or %NULL for auto-detection.
  * @error: Location to store error, or NULL
  *
@@ -515,42 +550,31 @@ gssdp_client_new (GMainContext *main_context,
                   const char   *iface,
                   GError      **error)
 {
+        if (main_context) {
+                g_warning ("GSSDPClient:main-context is deprecated."
+                           " Please use g_main_context_push_thread_default()");
+        }
+
         return g_initable_new (GSSDP_TYPE_CLIENT,
                                NULL,
                                error,
-                               "main-context", main_context,
                                "interface", iface,
                                NULL);
 }
 
-/**
- * Sets the GMainContext @client is associated with to @main_context
- **/
-static void
-gssdp_client_set_main_context (GSSDPClient  *client,
-                               GMainContext *main_context)
-{
-        g_return_if_fail (GSSDP_IS_CLIENT (client));
-
-        /* A NULL main_context is fine */
-        if (main_context)
-                client->priv->main_context = g_main_context_ref (main_context);
-
-        g_object_notify (G_OBJECT (client), "main-context");
-}
-
-/**
+/*
  * gssdp_client_get_main_context
  * @client: A #GSSDPClient
  *
  * Returns: (transfer none): The #GMainContext @client is associated with, or NULL.
+ * Deprecated: 0.11.2: Returns g_main_context_get_thread_default()
  **/
 GMainContext *
 gssdp_client_get_main_context (GSSDPClient *client)
 {
         g_return_val_if_fail (GSSDP_IS_CLIENT (client), NULL);
 
-        return client->priv->main_context;
+        return g_main_context_get_thread_default ();
 }
 
 /**
@@ -687,16 +711,17 @@ gssdp_client_get_active (GSSDPClient *client)
  * Sends @message to @dest_ip.
  **/
 void
-_gssdp_client_send_message (GSSDPClient *client,
-                            const char  *dest_ip,
-                            gushort      dest_port,
-                            const char  *message)
+_gssdp_client_send_message (GSSDPClient      *client,
+                            const char       *dest_ip,
+                            gushort           dest_port,
+                            const char       *message,
+                            _GSSDPMessageType type)
 {
         gssize res;
         GError *error = NULL;
-        GSocket *socket = NULL;
         GInetAddress *inet_address = NULL;
         GSocketAddress *address = NULL;
+        GSocket *socket;
 
         g_return_if_fail (GSSDP_IS_CLIENT (client));
         g_return_if_fail (message != NULL);
@@ -713,9 +738,16 @@ _gssdp_client_send_message (GSSDPClient *client,
         if (dest_port == 0)
                 dest_port = SSDP_PORT;
 
+        if (type == _GSSDP_DISCOVERY_REQUEST)
+                socket = gssdp_socket_source_get_socket
+                                        (client->priv->search_socket);
+        else
+                socket = gssdp_socket_source_get_socket
+                                        (client->priv->request_socket);
+
         inet_address = g_inet_address_new_from_string (dest_ip);
         address = g_inet_socket_address_new (inet_address, dest_port);
-        socket = gssdp_socket_source_get_socket (client->priv->request_socket);
+
         res = g_socket_send_to (socket,
                                 address,
                                 message,
@@ -734,9 +766,9 @@ _gssdp_client_send_message (GSSDPClient *client,
         g_object_unref (inet_address);
 }
 
-/**
+/*
  * Generates the default server ID
- **/
+ */
 static char *
 make_server_id (void)
 {
@@ -770,7 +802,9 @@ parse_http_request (char                *buf,
                     SoupMessageHeaders **headers,
                     int                 *type)
 {
-        char *req_method;
+        char *req_method = NULL;
+        char *path = NULL;
+        SoupHTTPVersion version;
 
         *headers = soup_message_headers_new (SOUP_MESSAGE_HEADERS_REQUEST);
 
@@ -778,8 +812,10 @@ parse_http_request (char                *buf,
                                         len,
                                         *headers,
                                         &req_method,
-                                        NULL,
-                                        NULL) == SOUP_STATUS_OK) {
+                                        &path,
+                                        &version) == SOUP_STATUS_OK &&
+            version == SOUP_HTTP_1_1 &&
+            (path && g_ascii_strncasecmp (path, "*", 1) == 0)) {
                 if (g_ascii_strncasecmp (req_method,
                                          SSDP_SEARCH_METHOD,
                                          strlen (SSDP_SEARCH_METHOD)) == 0)
@@ -793,11 +829,20 @@ parse_http_request (char                *buf,
 
                 g_free (req_method);
 
+                if (path)
+                        g_free (path);
+
                 return TRUE;
         } else {
                 soup_message_headers_free (*headers);
                 *headers = NULL;
 
+                if (path)
+                        g_free (path);
+
+                if (req_method)
+                        g_free (req_method);
+
                 return FALSE;
         }
 }
@@ -846,9 +891,9 @@ inet_netof (struct in_addr in) {
 }
 #endif
 
-/**
+/*
  * Called when data can be read from the socket
- **/
+ */
 static gboolean
 socket_source_cb (GSSDPSocketSource *socket_source, GSSDPClient *client)
 {
@@ -999,6 +1044,18 @@ multicast_socket_source_cb (GIOChannel  *source,
         return socket_source_cb (client->priv->multicast_socket, client);
 }
 
+static gboolean
+search_socket_source_cb (GIOChannel  *source,
+                         GIOCondition condition,
+                         gpointer     user_data)
+{
+        GSSDPClient *client;
+
+        client = GSSDP_CLIENT (user_data);
+
+        return socket_source_cb (client->priv->search_socket, client);
+}
+
 #ifdef G_OS_WIN32
 static gboolean
 is_primary_adapter (PIP_ADAPTER_ADDRESSES adapter)
@@ -1169,8 +1226,8 @@ get_host_ip (char **iface, char **network)
                         continue;
                 else if (!(ifa->ifa_flags & IFF_UP))
                         continue;
-		else if ((ifa->ifa_flags & IFF_POINTOPOINT))
-			continue;
+                else if ((ifa->ifa_flags & IFF_POINTOPOINT))
+                        continue;
 
                 /* Loopback and IPv6 interfaces go at the bottom on the list */
                 if (ifa->ifa_flags & IFF_LOOPBACK ||
